import 'dart:convert';
import 'dart:io';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:m2p_flutter_base/http/dio_client.dart';
import 'package:m2p_flutter_base/m2p_encrypt_decrypt_config.dart';

enum M2PNetworkingRequestType {
  get,
  post,
  put,
  delete,
  upload,
  download,
}

class M2PNetworkingRepository {
  //Progress sample model
  static void onReceiveProgress(int received, int total) {
    if (total != -1) {
      /*setState(() {
        _progress = (received / total * 100).toStringAsFixed(0) + "%";
      });*/
    }
  }

  static Future<Map<String, dynamic>> apiRequest(
      M2PNetworkingRequestType requestType, String urlString,
      {dynamic requestBodydata,
      List<File>? file,
      BuildContext? context,
      String? savePath,
      bool isConsiderAccessToken = true,
      bool isCanPop = false}) async {
    Map<String, dynamic> returnValue;

    if (ConnectivityResult.none != await Connectivity().checkConnectivity()) {
      if (!M2PKeyesConfiguration.isAPIRunning) {
        M2PKeyesConfiguration.isAPIRunning = true;
        debugPrint(
            'Before Request -->\n urlString-->$urlString \n  requestBodydata-->$requestBodydata');
        try {
          Response<dynamic>? response;
          switch (requestType) {
            case M2PNetworkingRequestType.get:
            case M2PNetworkingRequestType.delete:
              {
                response = requestType == M2PNetworkingRequestType.delete
                    ? await DioClient.dioConfig(
                            url: M2PKeyesConfiguration.baseUrl)
                        .delete(urlString)
                    : await DioClient.dioConfig(
                            url: M2PKeyesConfiguration.baseUrl)
                        .get(urlString);
                break;
              }
            case M2PNetworkingRequestType.upload:
              {
                final FormData data = FormData.fromMap({
                  'files': DioClient.listOfMultiPart(file!),
                  'data': requestBodydata
                });
                response = await DioClient.dioConfig(
                        url: M2PKeyesConfiguration.baseUrl)
                    .post(urlString, data: data);
                break;
              }

            case M2PNetworkingRequestType.download:
              {
                response = await DioClient.dioConfig(
                        url: M2PKeyesConfiguration.baseUrl)
                    .download(urlString, savePath,
                        onReceiveProgress: onReceiveProgress);
                break;
              }
            case M2PNetworkingRequestType.put:
              {
                response = await DioClient.dioConfig(
                        url: M2PKeyesConfiguration.baseUrl)
                    .put(
                  urlString,
                  data: requestBodydata,
                );
                break;
              }

            case M2PNetworkingRequestType.post:
            default:
              {
                response = await DioClient.dioConfig(
                        url: M2PKeyesConfiguration.baseUrl)
                    .post(urlString, data: requestBodydata);
              }
          }
          debugPrint(
              'After Request -->\n urlString-->$urlString \n  requestBodyData-->$requestBodydata'
              '\n  response-->${jsonDecode(response.toString())}');

          if (isConsiderAccessToken) {
            if (response!.headers[M2PDioAtttributeConfiguration.accessToken!] !=
                null) {
              debugPrint(
                  'Access Token is from as header=> ${response.headers['access-token']}');
              // Here get New Access Token for every API call then store
              M2PKeyesConfiguration.accessToken = response
                  .headers[M2PDioAtttributeConfiguration.accessToken!]![0]
                  .toString();
            }
          }
          returnValue = {
            'success': true,
            'data': response!.data,
            'statusCode': response.data[M2PDioAtttributeConfiguration.status],
          };
          debugPrint('Response--> $returnValue');
        } on DioError catch (e) {
          dynamic error;
          String? invalidAccessServerError;
          if (e.response != null) {
            error = DioClient.errorHandling(e);
          } else {
            error = M2PKeyesConfiguration
                .connectionTimeout; // connection timeout sometime will come
          }

          debugPrint(
              'Error Status :  urlString-->$urlString \n  requestBodydata-->$requestBodydata'
              '\n  response-->${jsonDecode(e.response.toString())}');

          if (error.toString() != 'DioErrorType.response') {
            /*isCanPop is used for if i load new api then get any error then pop the back screen*/
            if (isCanPop == true) {
              Navigator.pop(context!);
            }
            M2PKeyesConfiguration.apiErrorStatus(
                errorString: error.toString(), position: ToastGravity.CENTER);
          }

          if (e.response != null) {
            /*This is sample response model*/
            /*{
                  'status': 401,
                  "message": "Access token invalid and no refresh token provided",
                  "result": {}
               };*/

            if (e.response!.statusCode == 401) {
              //  here check accestoken expire or not after go to login
              invalidAccessServerError = e.response!.data[
                      M2PDioAtttributeConfiguration.responseMessageAttribute] ??
                  'Session Expired!';
              String? errVal;
              if (e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Error refreshing access token: Invalid refresh token' ||
                  e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Error getting KeyCloak session' ||
                  e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Error refreshing access token: Token is not active' ||
                  e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Daily token session expired' ||
                  e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Access token invalid and no refresh token provided' ||
                  e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ==
                      'Error refreshing access token: Session not active' ||
                  invalidAccessServerError == 'Session Expired!') {
                errVal = 'Logout triggered due to inactivity / another';
              }
              M2PKeyesConfiguration.apiErrorStatus(
                  errorString: errVal ??
                      e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute],
                  position: ToastGravity.BOTTOM);
            } else if (e.response!.statusCode == 502) {
              //  server not response --> api not working
              invalidAccessServerError =
                  M2PKeyesConfiguration.internalServerError;
              M2PKeyesConfiguration.apiErrorStatus(
                  errorString: M2PKeyesConfiguration.internalServerError,
                  position: ToastGravity.BOTTOM);
            } else if (e.response!.statusCode == 400) {
              //Event address not present
              M2PKeyesConfiguration.apiErrorStatus(
                  errorString: e.response!.data[M2PDioAtttributeConfiguration
                          .responseMessageAttribute] ??
                      '',
                  position: ToastGravity.BOTTOM);
            }
          }
          returnValue = {
            'success': false,
            'data': (invalidAccessServerError != null)
                ? invalidAccessServerError
                : e.response != null
                    ? e.response!.data
                    : error,
            'statusCode': e.response != null ? e.response!.statusCode : '',
          };
        }
      } else {
        //429 Too Many Requests
        /* The user has sent too many requests in a given amount of time (“rate limiting”). */
        returnValue = {
          'success': false,
          'data': 'No Internet Connection',
          'statusCode':
              '429', /* 12029 Cannot Connect, The attempt to connect to the server failed.*/
        };
        M2PKeyesConfiguration.apiErrorStatus(
            errorString:
                "System fetching the data... Please try again after some time",
            position: ToastGravity.BOTTOM);
      }
    } else {
      returnValue = {
        'success': false,
        'data': 'No Internet Connection',
        'statusCode':
            '12029', /* 12029 Cannot Connect, The attempt to connect to the server failed.*/
      };
    }
    //To reset the API permission
    M2PKeyesConfiguration.isAPIRunning = false;
    return returnValue;
  }
}
